@@ -1,7 +1,8 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-
  * 
- * Copyright (C) 2003-2004 Red Hat, Inc.
  * Copyright (C) 2003 Novell, Inc.
+ * Copyright (C) 2003-2004 Red Hat, Inc.
+ * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,13 +32,20 @@
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-vfs-extensions.h>
 #include <libnautilus-extension/nautilus-menu-provider.h>
+#include <libnautilus-extension/nautilus-location-widget-provider.h>
+
+#include "nautilus-burn-bar.h"
 
 #define NAUTILUS_TYPE_BURN  (nautilus_burn_get_type ())
 #define NAUTILUS_BURN(o)    (G_TYPE_CHECK_INSTANCE_CAST ((o), NAUTILUS_TYPE_BURN, NautilusBurn))
+#define NAUTILUS_IS_BURN(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), NAUTILUS_TYPE_BURN))
+
+typedef struct _NautilusBurnPrivate NautilusBurnPrivate;
 
 typedef struct 
 {
-	GObject parent_slot;
+	GObject              parent_slot;
+        NautilusBurnPrivate *priv;
 } NautilusBurn;
 
 typedef struct 
@@ -45,11 +53,67 @@ typedef struct
 	GObjectClass parent_slot;
 } NautilusBurnClass;
 
+#define NAUTILUS_BURN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NAUTILUS_TYPE_BURN, NautilusBurnPrivate))
+
+struct _NautilusBurnPrivate
+{
+	GnomeVFSMonitorHandle *burn_monitor;
+	guint                  empty : 1;
+
+	guint                  start_monitor_id;
+
+	GSList                *widget_list;
+};
+
 static GType nautilus_burn_get_type      (void);
 static void  nautilus_burn_register_type (GTypeModule *module);
 
 static GObjectClass *parent_class;
 
+#undef DEBUG_ENABLE
+
+#ifdef DEBUG_ENABLE
+#ifdef G_HAVE_ISO_VARARGS
+#  define DEBUG_PRINT(...) debug_print (__VA_ARGS__);
+#elif defined(G_HAVE_GNUC_VARARGS)
+#  define DEBUG_PRINT(args...) debug_print (args);
+#endif
+#else
+#ifdef G_HAVE_ISO_VARARGS
+#  define DEBUG_PRINT(...)
+#elif defined(G_HAVE_GNUC_VARARGS)
+#  define DEBUG_PRINT(args...)
+#endif
+#endif
+
+
+#ifdef DEBUG_ENABLE
+static FILE *debug_out = NULL;
+
+static void
+debug_init (void)
+{
+	const char path [50] = "burn_extension_debug_XXXXXX";
+	int  fd = g_file_open_tmp (path, NULL, NULL);
+	if (fd >= 0) {
+		debug_out = fdopen (fd, "a");
+	}
+}
+
+static void
+debug_print (const char *format, ...)
+{
+	va_list args;
+
+	va_start (args, format);
+	vfprintf ((debug_out ? debug_out : stderr), format, args);
+	vfprintf (stderr, format, args);
+	va_end (args);
+	if (debug_out)
+		fflush (debug_out);
+}
+#endif
+
 static void
 launch_process (char **argv, GtkWindow *parent)
 {
@@ -65,38 +129,41 @@ launch_process (char **argv, GtkWindow *parent)
 
 		eel_show_error_dialog (_("Unable to launch the cd burner application"),
 				       error->message,
-				       _("Can't launch cd burner"),
 				       GTK_WINDOW (parent));
 		g_error_free (error);
 	}
 }
 
 static void
-write_activate_cb (NautilusMenuItem *item,
-		   gpointer          user_data)
+launch_ncb_on_window (GtkWindow *window)
 {
 	char *argv [2];
 
 	argv [0] = g_build_filename (BINDIR, "nautilus-cd-burner", NULL);
 	argv [1] = NULL;
 
-	launch_process (argv, GTK_WINDOW (user_data));
+	launch_process (argv, window);
 
 	g_free (argv [0]);
 }
 
+static void
+write_activate_cb (NautilusMenuItem *item,
+		   gpointer          user_data)
+{
+	launch_ncb_on_window (GTK_WINDOW (user_data));
+}
+
 static void
 write_iso_activate_cb (NautilusMenuItem *item,
 		       gpointer          user_data)
 {
 	NautilusFileInfo *file;
-	GtkWidget        *window;
 	char             *argv [3];
 	char             *uri;
 	char             *image_name;
 
 	file = g_object_get_data (G_OBJECT (item), "file");
-	window = g_object_get_data (G_OBJECT (item), "window");
 
 	uri = nautilus_file_info_get_uri (file);
 	image_name = gnome_vfs_get_local_path_from_uri (uri);
@@ -124,13 +191,11 @@ write_cue_activate_cb (NautilusMenuItem *item,
 		       gpointer          user_data)
 {
 	NautilusFileInfo *file;
-	GtkWidget        *window;
 	char             *argv [3];
 	char             *uri;
 	char             *image_name;
 
 	file = g_object_get_data (G_OBJECT (item), "file");
-	window = g_object_get_data (G_OBJECT (item), "window");
 
 	uri = nautilus_file_info_get_uri (file);
 	image_name = gnome_vfs_get_local_path_from_uri (uri);
@@ -157,13 +222,9 @@ static void
 copy_disc_activate_cb (NautilusMenuItem *item,
 		       gpointer          user_data)
 {
-	NautilusFileInfo *file;
-	GtkWidget        *window;
 	char             *argv [3];
 	char             *device_path;
 
-	file = g_object_get_data (G_OBJECT (item), "file");
-	window = g_object_get_data (G_OBJECT (item), "window");
 	device_path = g_object_get_data (G_OBJECT (item), "drive_device_path");
 
 	if (!device_path) {
@@ -313,28 +374,9 @@ nautilus_burn_get_toolbar_items (NautilusMenuProvider *provider,
 				 NautilusFileInfo     *current_folder)
 {
 	GList *items;
-	char  *scheme;
 
 	items = NULL;
 
-	scheme = nautilus_file_info_get_uri_scheme (current_folder);
-
-	if (!strcmp (scheme, "burn")) {
-		NautilusMenuItem *item;
-		
-		item = nautilus_menu_item_new ("NautilusBurn::write_toolbar",
-					       _("Write to Disc"),
-					       _("Write contents to a CD or DVD disc"),
-					       "gnome-dev-cdrom");
-		g_object_set (item, "priority", TRUE, NULL);
-		g_signal_connect (item, "activate", 
-				  G_CALLBACK (write_activate_cb), window);
-		
-		items = g_list_append (items, item);
-	}
-
-	g_free (scheme);
-
 	return items;
 }
 
@@ -346,15 +388,233 @@ nautilus_burn_menu_provider_iface_init (NautilusMenuProviderIface *iface)
 	iface->get_toolbar_items = nautilus_burn_get_toolbar_items;
 }
 
-static void 
-nautilus_burn_instance_init (NautilusBurn *cvs)
+static void
+bar_activated_cb (NautilusBurnBar *bar,
+		  gpointer         data)
 {
+	launch_ncb_on_window (GTK_WINDOW (data));
+}
+
+static gboolean
+dir_is_empty (const char *uri)
+{
+	GnomeVFSFileInfo	*info;
+	GnomeVFSDirectoryHandle *handle;
+	GnomeVFSResult		 result;
+	gboolean		 found_file;
+	
+	result = gnome_vfs_directory_open (&handle, uri, GNOME_VFS_FILE_INFO_DEFAULT);
+
+	if (result != GNOME_VFS_OK) {
+		DEBUG_PRINT ("Could not open burn uri %s: %s\n",
+			     uri,
+			     gnome_vfs_result_to_string (result));
+		return TRUE;
+	}
+
+	info = gnome_vfs_file_info_new ();
+
+	found_file = FALSE;
+	
+	while (TRUE) {
+		result = gnome_vfs_directory_read_next (handle, info);
+		if (result != GNOME_VFS_OK) {
+			break;
+		}
+
+		/* Skip "." and "..".  */
+		if (info->name [0] == '.'
+		    && (info->name [1] == 0
+			|| (info->name [1] == '.' && info->name [2] == 0))) {
+			gnome_vfs_file_info_clear (info);
+			continue;
+		}
+		
+		found_file = TRUE;
+		break;
+	}
+
+	gnome_vfs_directory_close (handle);
+	gnome_vfs_file_info_unref (info);
+
+	return !found_file;
 }
 
 static void
-nautilus_burn_class_init (NautilusBurnClass *class)
+destroyed_callback (GtkWidget    *widget,
+		    NautilusBurn *burn)
 {
-	parent_class = g_type_class_peek_parent (class);
+	burn->priv->widget_list = g_slist_remove (burn->priv->widget_list, widget);
+}
+
+static void
+sense_widget (NautilusBurn *burn,
+	      GtkWidget    *widget)
+{
+	gtk_widget_set_sensitive (widget, !burn->priv->empty);
+
+	burn->priv->widget_list = g_slist_prepend (burn->priv->widget_list, widget);
+
+	g_signal_connect (widget, "destroy",
+			  G_CALLBACK (destroyed_callback),
+			  burn);
+}
+
+static GtkWidget *
+nautilus_burn_get_location_widget (NautilusLocationWidgetProvider *iface,
+				   const char                     *uri,
+				   GtkWidget                      *window)
+{
+	if (g_str_has_prefix (uri, "burn:")) {
+		GtkWidget    *bar;
+		NautilusBurn *burn;
+
+		DEBUG_PRINT ("Get location widget for burn\n");
+
+		burn = NAUTILUS_BURN (iface);
+
+		bar = nautilus_burn_bar_new ();
+
+		sense_widget (burn, nautilus_burn_bar_get_button (NAUTILUS_BURN_BAR (bar)));
+
+		g_signal_connect (bar, "activate",
+				  G_CALLBACK (bar_activated_cb),
+				  window);
+
+		gtk_widget_show (bar);
+
+		return bar;
+	}
+
+	return NULL;
+}
+
+static void
+nautilus_burn_location_widget_provider_iface_init (NautilusLocationWidgetProviderIface *iface)
+{
+	iface->get_widget = nautilus_burn_get_location_widget;
+}
+
+static void
+update_widget_sensitivity (GtkWidget    *widget,
+			   NautilusBurn *burn)
+{
+	gtk_widget_set_sensitive (widget, !burn->priv->empty);
+}
+
+static void
+burn_monitor_cb (GnomeVFSMonitorHandle    *handle,
+		 const gchar              *text_uri,
+		 const gchar              *info_uri,
+		 GnomeVFSMonitorEventType  event_type,
+		 gpointer                  data)
+{
+	NautilusBurn *burn;
+	gboolean      is_empty;
+
+	burn = NAUTILUS_BURN (data);
+
+	is_empty = burn->priv->empty;
+
+	if (event_type == GNOME_VFS_MONITOR_EVENT_CREATED) {
+		is_empty = FALSE;
+	} else if (event_type == GNOME_VFS_MONITOR_EVENT_DELETED) {
+		is_empty = dir_is_empty ("burn:///");
+	}
+
+	if (burn->priv->empty != is_empty) {
+		burn->priv->empty = is_empty;
+		g_slist_foreach (burn->priv->widget_list, (GFunc)update_widget_sensitivity, burn);
+	}
+}
+
+static gboolean
+start_monitor (NautilusBurn *burn)
+{
+	GnomeVFSResult    result;
+	char             *directory;
+
+	directory = g_strdup ("burn:///");
+
+	DEBUG_PRINT ("Starting monitor for %s\n", directory);
+
+	result = gnome_vfs_monitor_add (&burn->priv->burn_monitor,
+					directory,
+					GNOME_VFS_MONITOR_DIRECTORY,
+					burn_monitor_cb,
+					burn);
+
+	if (result != GNOME_VFS_OK) {
+		DEBUG_PRINT ("Could not monitor directory %d: %s\n",
+			     directory,
+			     gnome_vfs_result_to_string (result));
+
+		burn->priv->burn_monitor = NULL;
+	}
+
+	burn->priv->empty = dir_is_empty (directory);
+	DEBUG_PRINT ("Init burn extension, empty: %d\n", burn->priv->empty);
+
+	g_free (directory);
+
+	burn->priv->start_monitor_id = 0;
+
+	return FALSE;
+}
+
+static void 
+nautilus_burn_instance_init (NautilusBurn *burn)
+{
+	burn->priv = NAUTILUS_BURN_GET_PRIVATE (burn);
+
+#ifdef DEBUG_ENABLE
+	debug_init ();
+#endif
+
+	burn->priv->start_monitor_id = g_timeout_add (1000,
+						      (GSourceFunc)start_monitor,
+						      burn);
+}
+
+static void
+nautilus_burn_finalize (GObject *object)
+{
+	NautilusBurn *burn;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (NAUTILUS_IS_BURN (object));
+
+	DEBUG_PRINT ("Finalizing burn extension\n");
+
+	burn = NAUTILUS_BURN (object);
+
+	g_return_if_fail (burn->priv != NULL);
+
+	if (burn->priv->start_monitor_id > 0) {
+		g_source_remove (burn->priv->start_monitor_id);
+	}
+
+	if (burn->priv->burn_monitor != NULL) {
+		gnome_vfs_monitor_cancel (burn->priv->burn_monitor);
+	}
+
+	if (burn->priv->widget_list != NULL) {
+		g_slist_free (burn->priv->widget_list);
+	}
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+static void
+nautilus_burn_class_init (NautilusBurnClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+	parent_class = g_type_class_peek_parent (klass);
+
+	object_class->finalize = nautilus_burn_finalize;
+
+	g_type_class_add_private (klass, sizeof (NautilusBurnPrivate));
 }
 
 static GType burn_type = 0;
@@ -385,6 +645,11 @@ nautilus_burn_register_type (GTypeModule *module)
 		NULL,
 		NULL
 	};
+	static const GInterfaceInfo location_widget_provider_iface_info = {
+		(GInterfaceInitFunc) nautilus_burn_location_widget_provider_iface_init,
+		NULL,
+		NULL
+	};
 
 	burn_type = g_type_module_register_type (module,
 						 G_TYPE_OBJECT,
@@ -395,6 +660,10 @@ nautilus_burn_register_type (GTypeModule *module)
 				     burn_type,
 				     NAUTILUS_TYPE_MENU_PROVIDER,
 				     &menu_provider_iface_info);
+	g_type_module_add_interface (module,
+				     burn_type,
+				     NAUTILUS_TYPE_LOCATION_WIDGET_PROVIDER,
+				     &location_widget_provider_iface_info);
 }
 
 void
