@@ -72,7 +72,8 @@ int    get_dvd_r_rw_profile (const char *name);
 int    get_mmc_profile      (int fd);
 gint64 get_disc_size_cd     (int fd);
 int    get_disc_size_cdr    (int fd);
-gint64 get_disc_size_dvd    (int fd, int mmc_profile);
+gint64 get_disc_size_dvd    (int fd);
+gint64 get_disc_size_dvdr   (int fd, int mmc_profile);
 int    get_read_write_speed (int fd, int *read_speed, int *write_speed);
 int    get_disc_status      (int fd, int *empty, int *is_rewritable, int *is_blank);
 
@@ -96,10 +97,7 @@ static struct {
 	gboolean can_write_cdrw;
 	gboolean can_write_dvdr;
 	gboolean can_write_dvdram;
-} recorder_whitelist [] = {
-	{ "IOMEGA - CDRW9602EXT-B", TRUE, TRUE, FALSE, FALSE },
-	{ "SONY - CD-R   CDU948S",  TRUE, FALSE, FALSE, FALSE },
-};
+} recorder_whitelist [] = {};
 
 typedef enum {
 	NAUTILUS_BURN_DRIVE_PROTOCOL_IDE,
@@ -385,6 +383,135 @@ add_whitelist (NautilusBurnDrive *drive)
 }
 #endif /* !__linux__ */
 
+/**
+ * nautilus_burn_drive_get_max_speed_write:
+ * @drive: #NautilusBurnDrive
+ *
+ * Get the maximum write speed that the drive is capable of
+ *
+ * Returns: The speed of the drive, in device units.
+ *
+ * Since: 2.14
+ **/
+int
+nautilus_burn_drive_get_max_speed_write (NautilusBurnDrive *drive)
+{
+	int speed;
+
+	g_return_val_if_fail (drive != NULL, -1);
+
+	speed = drive->max_speed_write;
+
+	return speed;
+}
+
+/**
+ * nautilus_burn_drive_get_max_speed_read:
+ * @drive: #NautilusBurnDrive
+ *
+ * Get the maximum read speed that the drive is capable of
+ *
+ * Returns: The speed of the drive, in device units.
+ *
+ * Since: 2.14
+ **/
+int
+nautilus_burn_drive_get_max_speed_read (NautilusBurnDrive *drive)
+{
+	int speed;
+
+	g_return_val_if_fail (drive != NULL, -1);
+
+	speed = drive->max_speed_read;
+
+	return speed;
+}
+
+/**
+ * nautilus_burn_drive_get_name_for_display:
+ * @drive: #NautilusBurnDrive
+ *
+ * Get the name of the drive for use in a user interface
+ *
+ * Returns: name of the drive.  Must be freed with g_free().
+ *
+ * Since: 2.14
+ **/
+char *
+nautilus_burn_drive_get_name_for_display (NautilusBurnDrive *drive)
+{
+	char *name;
+
+	g_return_val_if_fail (drive != NULL, NULL);
+
+	name = g_strdup (drive->display_name);
+
+	return name;
+}
+
+/**
+ * nautilus_burn_drive_can_rewrite:
+ * @drive: #NautilusBurnDrive
+ *
+ * Report the whether the drive is capable of re-recording
+ *
+ * Returns: %TRUE if the drive can rewrite, otherwise return %FALSE.
+ *
+ * Since: 2.14
+ **/
+gboolean
+nautilus_burn_drive_can_rewrite (NautilusBurnDrive *drive)
+{
+	int      type;
+	gboolean can_rewrite;
+
+	g_return_val_if_fail (drive != NULL, FALSE);
+
+	type = drive->type;
+
+	can_rewrite = (type & NAUTILUS_BURN_DRIVE_TYPE_CDRW_RECORDER
+		       || type & NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER
+		       || type & NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_RW_RECORDER);
+
+	return can_rewrite;
+}
+
+/**
+ * nautilus_burn_drive_can_write:
+ * @drive: #NautilusBurnDrive
+ *
+ * Report the whether the drive is capable of recording
+ *
+ * Returns: %TRUE if the drive is a recorder, otherwise return %FALSE.
+ *
+ * Since: 2.14
+ **/
+gboolean
+nautilus_burn_drive_can_write (NautilusBurnDrive *drive)
+{
+	int      type;
+	gboolean can_write;
+	gboolean can_rewrite;
+
+	g_return_val_if_fail (drive != NULL, FALSE);
+
+	type = drive->type;
+
+	/* Handle rewritables first */
+	can_rewrite = nautilus_burn_drive_can_rewrite (drive);
+	if (can_rewrite)
+		return TRUE;
+
+	/* now only handle non-rewritables */
+	can_write = (type & NAUTILUS_BURN_DRIVE_TYPE_FILE
+		     || type & NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER
+		     || type & NAUTILUS_BURN_DRIVE_TYPE_DVD_RAM_RECORDER
+		     || type & NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_RECORDER
+		     || type & NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_DL_RECORDER);
+
+	return can_write;
+}
+
 static void
 add_dvd_plus (NautilusBurnDrive *drive)
 {
@@ -664,25 +791,22 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 			int blank, rewrite, empty;
 
 			if (get_disc_status (fd, &empty, &rewrite, &blank) == 0) {
-				close_ioctl_handle (ioctl_handle);
 
 				if (is_rewritable)
 					*is_rewritable = rewrite;
+				
 				if (is_blank)
 					*is_blank = blank;
-
-				disc_type = get_disc_type (device);
-				disc_type_get_info (disc_type, is_blank, has_data, has_audio);
-
-				if (empty)
-					return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
-				else
-					return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
 			}
-
 			close_ioctl_handle (ioctl_handle);
 
-			return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
+			disc_type = get_disc_type (device);
+			disc_type_get_info (disc_type, is_blank, has_data, has_audio);
+
+			if (empty)
+				return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
+			else
+				return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
 		}
 	}
 
@@ -804,14 +928,14 @@ nautilus_burn_drive_hal_get_media_type_full (NautilusBurnDrive *drive,
 				dbus_error_free (&error);
 				libhal_free_string_array (device_names);
 
-				return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
+				goto fallback;
 			}
 
 			if (num_devices <= 0) {
 				g_warning ("No HAL devices found for UDI '%s'", drive->priv->udi);
 				libhal_free_string_array (device_names);
 
-				return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
+				goto fallback;
 			}
 
 			/* just look at the first child */
@@ -884,10 +1008,16 @@ nautilus_burn_drive_hal_get_media_type_full (NautilusBurnDrive *drive,
 
 			libhal_free_string_array (device_names);
 
-			return type;
+			/* if the type has been determined using HAL, return it.
+			   Otherwise, fall through and try the non-HAL approach. */
+			if (type != NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN
+			    && type != NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
+				return type;
+			}
 		}
 	}
 
+ fallback:
 	return nautilus_burn_drive_get_media_type_from_path_full (drive->device,
 								  is_rewritable,
 								  is_blank,
@@ -982,6 +1112,9 @@ nautilus_burn_drive_get_media_size_from_path (const char *device)
 		else
 			size = 0;
 		break;
+	case 0x10:
+		size = get_disc_size_dvd (fd);
+		break;
 	case 0x11:
 	case 0x13:
 	case 0x14:
@@ -989,7 +1122,7 @@ nautilus_burn_drive_get_media_size_from_path (const char *device)
 	case 0x2B:
 	case 0x1A:
 	case 0x12:
-		size = get_disc_size_dvd (fd, mmc_profile);
+		size = get_disc_size_dvdr (fd, mmc_profile);
 		break;
 	default:
 		size = NAUTILUS_BURN_MEDIA_SIZE_NA;
@@ -1042,25 +1175,25 @@ try_media_size_from_type (NautilusBurnDrive *drive)
 			size = 0;
 		break;
 	case NAUTILUS_BURN_MEDIA_TYPE_DVD:
-		size = get_disc_size_dvd (fd, 0x10);
+		size = get_disc_size_dvd (fd);
 		break;
 	case NAUTILUS_BURN_MEDIA_TYPE_DVDR:
-		size = get_disc_size_dvd (fd, 0x11);
+		size = get_disc_size_dvdr (fd, 0x11);
 		break;
 	case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
-		size = get_disc_size_dvd (fd, 0x13);
+		size = get_disc_size_dvdr (fd, 0x13);
 		break;
 	case NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM:
-		size = get_disc_size_dvd (fd, 0x12);
+		size = get_disc_size_dvdr (fd, 0x12);
 		break;
 	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R:
-		size = get_disc_size_dvd (fd, 0x1B);
+		size = get_disc_size_dvdr (fd, 0x1B);
 		break;
 	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
-		size = get_disc_size_dvd (fd, 0x1A);
+		size = get_disc_size_dvdr (fd, 0x1A);
 		break;
 	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL:
-		size = get_disc_size_dvd (fd, 0x2B);
+		size = get_disc_size_dvdr (fd, 0x2B);
 		break;
 	default:
 		break;
